/*=========================================================================
 
  Program:   Orbit Visualization Tool
  Source:    $Source: /stor/devel/ovt2g/ovt/beans/SliderPropertyEditor.java,v $
  Date:      $Date: 2002/04/10 13:01:26 $
  Version:   $Revision: 2.4 $
 
 
Copyright (c) 2000-2002 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev,
Yuri Khotyaintsev)
All rights reserved.
 
Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:
 
 * No part of the software can be included in any commercial package without
written consent from the OVT team.
 
 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.
 
OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev
 
=========================================================================*/

/*
 * SliderEditor.java
 *
 * Created on 23  2001 ., 13:35
 */

package ovt.beans;

import ovt.util.*;
import ovt.interfaces.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

/**
 * This is a ComponentPropertyEditor with a JSlider component, can edit int and double properties.
 * The class uses value transformation chain: value <-> x <-> sliderPos
 * That's why there are different limits for value, x, pos.
 * @author  ko
 * @version
 */
public class SliderPropertyEditor extends ComponentPropertyEditor implements PropertyChangeListener {
    
    /** slider == component */
    private JSlider slider = null; 
    
    private ChangeListener changeListener;
    
    /** Holds value of the minimal value the property. */
    private double minimumValue = 0.;
    
    /** Holds value of the maximum value of the property. */
    private double maximumValue =  100.;
   
    /** This is a interval in points between small ticks on JSlider.
     * Is measured in slider's coordinates.
     */
    private int minorTickSpacing = 1;
    
    /** This is a interval in points between big ticks on JSlider.
     * Is measured in slider's coordinates.
     */
    private int majorTickSpacing = 1;
   
    /** Indicates if the labels have to be shown. */
    private boolean labeled = false;
    
    /** if true - the edited property is double, if not - int */
    protected boolean isDoubleEditor = false;
    
    /** Holds value of property numberOfMajorLabels. By default - 2.*/
    private int numberOfMajorLabels = 2; 
    

    /** Creates new linear SliderPropertyEditor without labels. */
    public SliderPropertyEditor(BasicPropertyDescriptor pd, double minValue, double maxValue) {
        super(pd);
        initialize(minValue, maxValue);
    }

    /** Creates new linear SliderPropertyEditor without labels. */
    public SliderPropertyEditor(BasicPropertyDescriptor pd, double minValue, double maxValue, int numberOfMajorLabels) {
        super(pd);
        initialize(minValue, maxValue);
        setNumberOfMajorLabels(numberOfMajorLabels);
        setLabeled(true);
    }
    
    /** Creates new linear SliderPropertyEditor */
    public SliderPropertyEditor(BasicPropertyDescriptor pd, int minValue, int maxValue, int majorTickSpacing) {
        super(pd);
        initialize( minValue, maxValue);
        setMinorTickSpacing(majorTickSpacing);
        setMajorTickSpacing(majorTickSpacing);
        setNumberOfMajorLabels((maxValue- minValue) / majorTickSpacing + 1);
    }
    
    /** Creates new linear SliderPropertyEditor */
    public SliderPropertyEditor(BasicPropertyDescriptor pd, int minValue, int maxValue, int majorTickSpacing, int minorTickSpacing) {
        super(pd);
        initialize( minValue, maxValue);
        setMinorTickSpacing(minorTickSpacing);
        setMajorTickSpacing(majorTickSpacing);
        setNumberOfMajorLabels((maxValue- minValue) / majorTickSpacing + 1);
        setLabeled(true);
    }

    
    private void initialize(double minValue, double maxValue) {
        // check for the property type
        String srcType = getPropertyDescriptor().getWriteMethod().getParameterTypes()[0].getName();
        setMinimumValue(minValue);
        setMaximumValue(maxValue);
        if (srcType.equals("double"))
            isDoubleEditor = true;
            
    }

    protected double pos2x(int pos) {
        return getMinX() +  ((double)(pos - getMinimumSliderValue())) * (getMaxX() - getMinX()) / ((double)(getMaximumSliderValue() - getMinimumSliderValue()));
    }
    
    protected int x2pos(double x) {
        return (int)Math.round((double)(getMaximumSliderValue() - getMinimumSliderValue())/(getMaxX() - getMinX())*(x - getMinX())) + getMinimumSliderValue();
    }
    
    /** linear dependence */
    protected double value2x( double value) {
        return value;
    }
    
    /** linear dependence */
    protected double x2value( double x) {
        return x;
    }

    protected double calcValueFromSliderPosition(int pos) {
        return x2value(pos2x(pos));
    }
    
    protected int calcSliderPositionFromValue(double value) {
        return x2pos(value2x(value));
    }
    
    private double getValueFromSlider() {
        return calcValueFromSliderPosition(getSliderPosition());
    }
    
    private int getSliderPosition() {
        return ((JSlider)component).getValue();
    }
    
    private int calcSliderPositionFromValue() {
        return calcSliderPositionFromValue(getValueAsDouble());
    }
    
    
    /** To get the value of the property one should use this method.
     * If the prperty is int - it is casted to double.
     */
    double getValueAsDouble() {
        Object obj = getValue();
        if (obj instanceof Integer) {
            return (double)((Integer) obj).intValue();
        } else if (obj instanceof Double) {
            return ((Double) obj).doubleValue();
        }
        return -1;
    }

    /** Getter for property min.
     * @return Value of property min.
     */
    public double getMinimumValue() {
        return minimumValue;
    }
    
    /** Setter for property min. Also set's minX
     * @param min New value of property min.
     */
    public void setMinimumValue(double min) {
        this.minimumValue = min;
    }
    
    /** Getter for property max.
     * @return Value of property max.
     */
    public double getMaximumValue() {
        return maximumValue;
    }
    
    /** Setter for property max. Also set's MaximumSliderValue
     * @param max New value of property max.
     */
    public void setMaximumValue(double max) {
        this.maximumValue = max;
    }
    
    public double getMinX() {
        return value2x(getMinimumValue());
    }
    
    public double getMaxX() {
        return value2x(getMaximumValue());
    }
    
    


        /** Getter for property minorTickSpacing.
     * @return Value of property minorTickSpacing.
     */
    public int getMinorTickSpacing() {
        return minorTickSpacing;
    }
    
    /** Setter for property minorTickSpacing.
     * @param minorTickSpacing New value of property minorTickSpacing.
 */
    public void setMinorTickSpacing(int minorTickSpacing) {
        this.minorTickSpacing = minorTickSpacing;
    }
    
    /** Getter for property majorTickSpacing.
     * @return Value of property majorTickSpacing.
     */
    public int getMajorTickSpacing() {
        return majorTickSpacing;
    }
    
    /** Setter for property majorTickSpacing.
     * @param majorTickSpacing New value of property majorTickSpacing.
     */
    public void setMajorTickSpacing(int majorTickSpacing) {
        this.majorTickSpacing = majorTickSpacing;
    }
    
/** Getter for property minimumSliderValue.
 * @return Value of property minimumSliderValue.
 */
    protected int getMinimumSliderValue() {
        return 0;
    }
    
/** Getter for property maximumSliderValue.
 * @return Value of property maximumSliderValue.
 */
    protected int getMaximumSliderValue() {
        if (isDoubleEditor) {
            if (isLabeled()) 
                return (getNumberOfMajorLabels() - 1) * getMajorTickSpacing();
            else 
                return 100;
        } else  // int editor
            return (int)(maximumValue - minimumValue) ;
        
    }
    
    
    protected String getLabel(double x) {
        if (isDoubleEditor)
            return "" + Math.round(x2value(x)*1000.)/1000.;
        else
            return "" + (int)x2value(x);
    }
    
    /** Getter for property numberOfMajorLabels.
     * @return Value of property numberOfMajorLabels.
 */
    protected int getNumberOfMajorLabels() {
        return numberOfMajorLabels;
    }
    
    /** Setter for property numberOfMajorLabels.
     * @param numberOfMajorLabels New value of property numberOfMajorLabels.
 */
    protected void setNumberOfMajorLabels(int numberOfMajorLabels) {
        int oldNumberOfMajorLabels = this.numberOfMajorLabels;
        this.numberOfMajorLabels = numberOfMajorLabels;
        propertySupport.firePropertyChange ("numberOfMajorLabels", new Integer (oldNumberOfMajorLabels), new Integer (numberOfMajorLabels));
    }

    
    
    private void setLabels() {
        if (slider == null) return;
        
        if (isLabeled()) {
            slider.setMinorTickSpacing(getMinorTickSpacing());
            slider.setMajorTickSpacing(getMajorTickSpacing());
            slider.setPaintTicks(true);
            slider.setPaintLabels(true);
            slider.setSnapToTicks(snapToTicks());
            
            Dictionary labels = new Hashtable();
            //double step = (getMaximumValue() - getMinimumValue())/((double)getNumberOfMajorLabels() - 1.);
            //
            double minX = getMinX();
            double maxX = getMaxX();
            
            //for (int i=getMinimumSliderValue(); i<=getMaximumSliderValue(); i+=getMajorTickSpacing()) {
                //int i = calcSliderPositionFromValue(v);
            //for(double v = getMinimumValue(); v<=getMaximumValue(); v+=step) {
            for (double n=0; n<getNumberOfMajorLabels(); n++) {
                double x = minX + (n/((double)getNumberOfMajorLabels()-1.))*(maxX - minX);
            
                labels.put(new Integer(x2pos(x)), new JLabel(getLabel(x)));
                //Log.log(""+i+"\t"+getLabel(i));
            }
            slider.setLabelTable(labels);
        }
        else {
            slider.setPaintTicks(false);
            slider.setPaintLabels(false);
            slider.setSnapToTicks(false);
        }
    }

        
    
    public Component getComponent() {
        if (component == null) {
            slider = new JSlider();
            // set the component
            component = slider;
            // create slider change listener
            changeListener = new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    //Log.log("Slider value :"+getValueFromSlider()+" pos="+getSliderPosition());
                    try {
                        if (isDoubleEditor)   
                            setValue(new Double(getValueFromSlider()));
                        else // integer
                            setValue(new Integer((int)Math.round(getValueFromSlider())));
                        
                        fireEditingFinished();
                    } catch (PropertyVetoException ignore) {}
                }
            };
            slider.addChangeListener(changeListener);
            updateSlider();
        }
        return component;
    }
    
    private void updateSlider() {
        if (slider != null) {
            slider.removeChangeListener(changeListener);
            slider.setMinimum(getMinimumSliderValue());
            //slider.setMaximum(isLinear() ? maximumSliderValue : maximumSliderValue+1);
            //Log.log("Setting max value for slider="+getMaximumSliderValue());
            slider.setMaximum(getMaximumSliderValue());
            slider.setValue(calcSliderPositionFromValue());
            setLabels();
            slider.addChangeListener(changeListener);
        }
    }
    
    /** Getter for property labeled.
     * @return Value of property labeled.
     */
    public boolean isLabeled() {
        return labeled;
    }
    
    /** Setter for property labeled.
     * @param labeled New value of property labeled.
     */
    public void setLabeled(boolean labeled) {
        this.labeled = labeled;
    }
    
    public void propertyChange(PropertyChangeEvent evt) {
        //Log.log("event with value = "+getValueAsDouble());
        
        if (slider != null) {
            slider.removeChangeListener(changeListener);
            slider.setValue(calcSliderPositionFromValue());
            slider.addChangeListener(changeListener);
        }
        //super.propertyChange(evt);// ???
    }
    
    
    protected boolean snapToTicks() {
        if (isDoubleEditor) return false;
        else return true; 
    }
}
